@@ -1,10 +1,11 @@
 /*
  * pio operations for file fragments or programs
  *
- * $Id: gfs_pio_section.c 10335 2016-12-03 21:57:15Z n-soda $
+ * $Id$
  */
 
 #include <assert.h>
+#include <stddef.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
@@ -794,6 +795,7 @@ gfs_file_section_context_alloc(void)
 	return (vc);
 }
 
+/* internal function: do not call external functions */
 gfarm_error_t
 gfs_pio_internal_set_view_section(GFS_File gf, char *host)
 {
@@ -872,7 +874,7 @@ gfs_pio_internal_set_view_section(GFS_File gf, char *host)
 		}
 
 		if (gf->open_flags & GFARM_FILE_APPEND) {
-			e = gfs_pio_seek(gf, 0, SEEK_END, NULL);
+			e = gfs_pio_seek_unlocked(gf, 0, SEEK_END, NULL);
 			if (e == GFARM_ERR_NO_ERROR)
 				goto finish;
 			(*vc->ops->storage_close)(gf);
@@ -1151,7 +1153,7 @@ gfs_pio_set_view_section(GFS_File gf, const char *section,
 	/* XXX - need to figure out ignorable error or not */
 
 	if (gf->open_flags & GFARM_FILE_APPEND)
-		e = gfs_pio_seek(gf, 0, SEEK_END, NULL);
+		e = gfs_pio_seek_unlocked(gf, 0, SEEK_END, NULL);
 
 storage_close:
 	if (e != GFARM_ERR_NO_ERROR)
@@ -1275,13 +1277,29 @@ gfarm_redirect_file(int fd, char *file, GFS_File *gfp)
 
 #endif /* not yet in gfarm v2 */
 
+struct gfs_profile_list section_profile_items[] = {
+	{ "set_view_section", "gfs_pio_set_view_section  : %g sec", "%g", 'd',
+	  offsetof(struct gfarm_gfs_pio_section_static, set_view_section_time)
+	},
+	{ "open_local_count", "gfs_pio_open_local_count  : %lld", "%llu", 'l',
+	  offsetof(struct gfarm_gfs_pio_section_static, open_local_count) },
+	{ "open_remote_count", "gfs_pio_open_remote_count : %lld", "%llu", 'l',
+	  offsetof(struct gfarm_gfs_pio_section_static, open_remote_count) },
+};
+
 void
 gfs_pio_section_display_timers(void)
 {
-	gflog_info(GFARM_MSG_1000113, "gfs_pio_set_view_section  : %f sec",
-		staticp->set_view_section_time);
-	gflog_info(GFARM_MSG_1003712, "gfs_pio_open_local_count  : %lld",
-		(unsigned long long)staticp->open_local_count);
-	gflog_info(GFARM_MSG_1003713, "gfs_pio_open_remote_count : %lld",
-		(unsigned long long)staticp->open_remote_count);
+	int n = GFARM_ARRAY_LENGTH(section_profile_items);
+
+	gfs_profile_display_timers(n, section_profile_items, staticp);
+}
+
+gfarm_error_t
+gfs_pio_section_profile_value(const char *name, char *value, size_t *sizep)
+{
+	int n = GFARM_ARRAY_LENGTH(section_profile_items);
+
+	return (gfs_profile_value(name, n, section_profile_items,
+		    staticp, value, sizep));
 }
