@@ -5,12 +5,15 @@
 #include <string.h>
 #include <stdio.h>
 #include <errno.h>
+#include <time.h>
+#include <sys/types.h>
 
 #include <gfarm/gfarm.h>
 
 #include "gfutil.h"
 #include "hash.h"
 
+#include "quota_info.h"
 #include "context.h"
 #include "gfp_xdr.h"
 #include "auth.h"
@@ -30,7 +33,7 @@ struct group {
 	char *groupname;
 	struct group_assignment users;
 	struct quota quota;
-	struct usage usage_tmp;
+	struct gfarm_quota_subject_info usage_tmp;
 	int invalid;	/* set when deleted */
 };
 
@@ -200,6 +203,9 @@ group_remove_internal(const char *groupname, int update_quota)
 	}
 	if (update_quota)
 		quota_group_remove(g);
+
+	/* NOTE: this group remains in struct inode */
+
 	/*
 	 * do not purge the hash entry.  Instead, invalidate it so
 	 * that it can be activated later.
@@ -266,6 +272,12 @@ group_name(struct group *g)
 	    g->groupname : REMOVED_GROUP_NAME);
 }
 
+char *
+group_name_even_invalid(struct group *g)
+{
+	return (g != NULL ? g->groupname : REMOVED_GROUP_NAME);
+}
+
 char *
 group_name_with_invalid(struct group *g)
 {
@@ -278,18 +290,19 @@ group_quota(struct group *g)
 	return (&g->quota);
 }
 
-struct usage *
+struct gfarm_quota_subject_info *
 group_usage_tmp(struct group *g)
 {
 	return (&g->usage_tmp);
 }
 
 void
-group_all(void *closure, void (*callback)(void *, struct group *),
-	  int valid_only)
+group_foreach(void *closure, void (*callback)(void *, struct group *),
+	int flags)
 {
 	struct gfarm_hash_iterator it;
 	struct group **g;
+	int valid_only = (flags & GROUP_FOREARCH_FLAG_VALID_ONLY) != 0;
 
 	for (gfarm_hash_iterator_begin(group_hashtab, &it);
 	     !gfarm_hash_iterator_is_end(&it);
@@ -428,9 +441,6 @@ void
 group_init(void)
 {
 	gfarm_error_t e;
-	struct group *admin;
-	struct gfarm_group_info gi;
-	static const char diag[] = "group_init";
 
 	group_hashtab =
 	    gfarm_hash_table_alloc(GROUP_HASHTAB_SIZE,
@@ -442,6 +452,15 @@ group_init(void)
 	if (e != GFARM_ERR_NO_ERROR)
 		gflog_error(GFARM_MSG_1000248,
 		    "loading groups: %s", gfarm_error_string(e));
+}
+
+void
+group_initial_entry(void)
+{
+	gfarm_error_t e;
+	struct group *admin;
+	struct gfarm_group_info gi;
+	static const char diag[] = "group_init";
 
 	if ((admin = group_lookup(ADMIN_GROUP_NAME)) == NULL) {
 		gflog_info(GFARM_MSG_1000249,
@@ -792,6 +811,10 @@ gfm_server_group_info_set(struct peer *peer, int from_client, int skip)
 		gflog_debug(GFARM_MSG_1001537,
 			"group_user_check() failed: %s",
 			gfarm_error_string(e));
+	} else if (gfarm_read_only_mode()) {
+		gflog_debug(GFARM_MSG_UNFIXED, "%s (%s@%s) during read_only",
+		    diag, peer_get_username(peer), peer_get_hostname(peer));
+		e = GFARM_ERR_READ_ONLY_FILE_SYSTEM;
 	/*
 	 * We have to call this before group_info_add(),
 	 * because group_info_add() frees the memory of gi
@@ -878,10 +901,14 @@ gfm_server_group_info_modify(struct peer *peer, int from_client, int skip)
 		gflog_debug(GFARM_MSG_1001541,
 			"group_lookup() failed");
 		e = GFARM_ERR_NO_SUCH_GROUP;
-	} else if ((e = group_user_check(&gi, diag)) != GFARM_ERR_NO_ERROR)
-		gflog_debug(GFARM_MSG_1001542,
-			"group_user_check() failed: %s", gfarm_error_string(e));
-	else {
+	} else if ((e = group_user_check(&gi, diag)) != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1001542, "group_user_check() failed: %s",
+		    gfarm_error_string(e));
+	} else if (gfarm_read_only_mode()) {
+		gflog_debug(GFARM_MSG_UNFIXED, "%s (%s@%s) during read_only",
+		    diag, peer_get_username(peer), peer_get_hostname(peer));
+		e = GFARM_ERR_READ_ONLY_FILE_SYSTEM;
+	} else {
 		group_modify(group, &gi, diag);
 		/* change all entries */
 		e = db_group_modify(&gi, 0, 0, NULL, 0, NULL);
@@ -945,6 +972,10 @@ gfm_server_group_info_remove(struct peer *peer, int from_client, int skip)
 		    "%s: administrator group \"%s\" should not be deleted",
 		    diag, groupname);
 		e = GFARM_ERR_OPERATION_NOT_PERMITTED;
+	} else if (gfarm_read_only_mode()) {
+		gflog_debug(GFARM_MSG_UNFIXED, "%s (%s@%s) during read_only",
+		    diag, peer_get_username(peer), peer_get_hostname(peer));
+		e = GFARM_ERR_READ_ONLY_FILE_SYSTEM;
 	} else
 		e = group_info_remove(groupname, diag);
 	free(groupname);
