@@ -13,10 +13,12 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <errno.h>
+#include <sys/types.h>
 #include <sys/socket.h>
 #include <signal.h> /* for sig_atomic_t */
 #include <netinet/in.h>
 #include <netdb.h> /* for NI_MAXHOST, NI_NUMERICHOST, etc */
+#include <time.h>
 
 #include <gfarm/gflog.h>
 #include <gfarm/error.h>
@@ -364,7 +366,7 @@ peer_replicated(struct peer *peer,
 	gfarm_mutex_unlock(&peer->replication_mutex, diag, replication_diag);
 
 	if (e != GFARM_ERR_NO_ERROR) {
-		gflog_error(GFARM_MSG_1002410,
+		gflog_notice(GFARM_MSG_1002410,
 		    "orphan replication (%s, %lld:%lld): s=%d d=%d size:%lld "
 		    "maybe the connection had a problem?",
 		    host_name(host), (long long)ino, (long long)gen,
@@ -1037,7 +1039,7 @@ peer_free(struct peer *peer)
 		    &peer->u.client.jobs);
 	peer->u.client.jobs = NULL;
 
-	if (db_begin(diag) == GFARM_ERR_NO_ERROR)
+	if (!gfarm_read_only_mode() && db_begin(diag) == GFARM_ERR_NO_ERROR)
 		transaction = 1;
 
 	peer_unset_pending_new_generation(peer, GFARM_ERR_CONNECTION_ABORTED);
@@ -1221,6 +1223,7 @@ peer_set_host(struct peer *peer, char *hostname)
 {
 	struct host *h;
 	struct mdhost *m;
+	char *aux;
 
 	switch (peer->id_type) {
 	case GFARM_AUTH_ID_TYPE_SPOOL_HOST:
@@ -1256,10 +1259,24 @@ peer_set_host(struct peer *peer, char *hostname)
 	}
 
 	if (peer->hostname != NULL) {
+		gflog_info(GFARM_MSG_1004751, "%s: set to %s",
+		    peer->hostname, hostname);
 		free(peer->hostname);
 		peer->hostname = NULL;
 	}
-
+	aux = gflog_get_auxiliary_info();
+	if (aux != NULL) {
+		free(aux);
+		GFARM_MALLOC_ARRAY(aux,
+		    strlen(peer->username) + 1 + strlen(hostname) + 1);
+		if (aux == NULL) {
+			gflog_error(GFARM_MSG_1004752, "no memory: "
+			    "gflog_set_auxiliary_info(%s@%s)",
+			    peer->username, hostname);
+		} else
+			sprintf(aux, "%s@%s", peer->username, hostname);
+		gflog_set_auxiliary_info(aux);
+	}
 	gflog_debug(GFARM_MSG_1002772,
 	    "%s connected from %s",
 	    peer_get_service_name(peer), abstract_host_get_name(peer->host));
@@ -1386,6 +1403,21 @@ peer_add_pending_new_generation_by_cookie(
 	return (GFARM_ERR_NO_ERROR);
 }
 
+/* NOTE: caller of this function should acquire giant_lock as well */
+struct inode *
+peer_get_pending_new_generation_by_cookie(
+	struct peer *peer, gfarm_uint64_t cookie_id)
+{
+	struct pending_new_generation_by_cookie *cookie;
+
+	GFARM_HCIRCLEQ_FOREACH(cookie, peer->pending_new_generation_cookies,
+	    cookie_link) {
+		if (cookie->id == cookie_id)
+			return (cookie->inode);
+	}
+	return (NULL);
+}
+
 /* NOTE: caller of this function should acquire giant_lock as well */
 int
 peer_remove_pending_new_generation_by_cookie(
